const Payment = require('../models/paymentschema');
const ApiError = require('../utils/ApiError');
const httpStatus = require('../utils/httpStatus');


 const stripe = require('stripe')(process.env.STRIPE_TEST_SECRET_KEY);
// const stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);



const createCheckoutSession = async (req, res) => {


   try {
       const {
           productName,
           price,
           successUrl,
           failUrl,
           quantity
          
       } = req.body;


      
       // Add the extra field 'status' and use rest to include other fields except 'price'
       const paymentData = {
           productName: productName,
           price: parseInt(price)/100,
           status: "processing",
       };
      


       const payment = await Payment.create(paymentData);


       const session = await stripe.checkout.sessions.create({
           payment_method_types: ['card'],
           line_items: [{
               price_data: {
                   currency: 'hkd',
                   product_data: {
                       name: productName
                   },
                   unit_amount: price,
               },
               quantity: quantity,
           }],
           mode: 'payment',
           success_url: successUrl,
           cancel_url: failUrl,
           metadata: {  // adding metadata here
               price: String(price),
               productName: productName,
               bookingid: payment.id,
          
          
           }
       });


       res.json({ id: session.id });




   } catch (error) {
       res.status(500).send({ error: error.message });
       console.error(error);
   }
};




const handleSessionDetails = async (req, res) => {
   try {
       const session = await stripe.checkout.sessions.retrieve(req.query.sessionId);


       if (!session.metadata || !session.metadata.bookingid) {
           return res.status(404).json({ error: 'Metadata not found or booking ID missing' });
       }


       let payment = await Payment.findById(session.metadata.bookingid);


       let attempts = 0;
       const maxAttempts = 10;


       while (payment.status === 'processing' && attempts < maxAttempts) {
           await new Promise(resolve => setTimeout(resolve, 3000)); // Wait for 3 seconds
           payment = await Payment.findById(session.metadata.bookingid); // Refresh the payment status
           attempts++;
       }


       // If after maximum attempts, the status is still processing, you might want to handle this case specifically.
       if (payment.status === 'processing') {
           return res.status(408).json({ error: 'Payment processing took too long.' });
       }


       res.json({
           session,
           paymentStatus: payment.status
       });


   } catch (error) {
       res.status(500).json({ error: 'Failed to retrieve session details' });
   }
};
const handleStripeWebhook = async (req, res) => {
   const incomingEvent = req.body;


   try {
       // Fetch the event from Stripe to ensure it's genuine
       const eventFromStripe = await stripe.events.retrieve(incomingEvent.id);


       if (eventFromStripe.type === 'checkout.session.completed') {
           const session = eventFromStripe.data.object;
          
           if (!session?.metadata?.bookingid) {
               console.error('Booking ID missing in Stripe event.');
               return res.status(400).send('Booking ID missing in Stripe event.');
           }


           const payment = await Payment.findById(session.metadata.bookingid);
          
           if (!payment) {
           
               return res.status(404).send('Payment not found.');
           }


           if (payment.status != 'processing') {
      
               return res.status(400).send('Payment has already been handled.');
           }


           // Code ro run when payment is successful


           Object.assign(payment, {
               status: 'success',
               paymentid: session.payment_intent,
           });
           await payment.save();
           res.sendStatus(200);




         
       } else {
           res.sendStatus(400);
       }
   } catch (error) {
       console.error('Error handling Stripe webhook:', error);
       res.status(500).send('Internal Server Error.');
   }
};




module.exports = {
   createCheckoutSession,
   handleSessionDetails,
   handleStripeWebhook
};
